@@ -14,7 +14,6 @@ pub use self::{
     Purity::{MayBeImpure, Pure},
 };
 use crate::ident::IdentLike;
-use scoped_tls::scoped_thread_local;
 use std::{
     borrow::Cow,
     f64::{INFINITY, NAN},
@@ -23,9 +22,9 @@ use std::{
     ops::Add,
 };
 use swc_atoms::{js_word, JsWord};
-use swc_common::{
-    collections::AHashSet, errors::Handler, Mark, Span, Spanned, SyntaxContext, DUMMY_SP,
-};
+#[deprecated(since = "0.50", note = "Use `swc_common::errors::HANDLER` directly")]
+pub use swc_common::errors::HANDLER;
+use swc_common::{collections::AHashSet, Mark, Span, Spanned, SyntaxContext, DUMMY_SP};
 use swc_ecma_ast::*;
 use swc_ecma_visit::{
     noop_visit_mut_type, noop_visit_type, Node, Visit, VisitMut, VisitMutWith, VisitWith,
@@ -59,11 +58,34 @@ impl Visit for ThisVisitor {
     /// Don't recurse into fn
     fn visit_function(&mut self, _: &Function, _: &dyn Node) {}
 
+    /// Don't recurse into fn
+    fn visit_getter_prop(&mut self, n: &GetterProp, _: &dyn Node) {
+        n.key.visit_with(n, self);
+    }
+
+    /// Don't recurse into fn
+    fn visit_method_prop(&mut self, n: &MethodProp, _: &dyn Node) {
+        n.key.visit_with(n, self);
+        n.function.visit_with(n, self);
+    }
+
+    /// Don't recurse into fn
+    fn visit_setter_prop(&mut self, n: &SetterProp, _: &dyn Node) {
+        n.key.visit_with(n, self);
+        n.param.visit_with(n, self);
+    }
+
     fn visit_this_expr(&mut self, _: &ThisExpr, _: &dyn Node) {
         self.found = true;
     }
 }
 
+/// This does not recurse into a function if `this` is changed by it.
+///
+/// e.g.
+///
+///   - The body of an arrow expression is visited.
+///   - The body of a function expression is **not** visited.
 pub fn contains_this_expr<N>(body: &N) -> bool
 where
     N: VisitWith<ThisVisitor>,
@@ -212,7 +234,7 @@ pub trait ModuleItemLike: StmtLike {
     }
 }
 
-pub trait StmtLike: Sized + 'static {
+pub trait StmtLike: Sized + 'static + Send + Sync {
     fn try_into_stmt(self) -> Result<Stmt, Self>;
     fn as_stmt(&self) -> Option<&Stmt>;
     fn from_stmt(stmt: Stmt) -> Self;
@@ -1894,11 +1916,6 @@ impl<'a> UsageFinder<'a> {
     }
 }
 
-scoped_thread_local!(
-    /// Used for error reporting in transform.
-    pub static HANDLER: Handler
-);
-
 /// make a new expression which evaluates `val` preserving side effects, if any.
 pub fn preserve_effects<I>(span: Span, val: Expr, exprs: I) -> Expr
 where
